/*
 *  File: TERM1.C
 *  Project: Terminal program in "C" for Comtrol Controllers
 *  Comments: the file TERMERR.ASM must also be linked in with this file
 *  Date: 18 October 1990
*/

#include <stdio.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <dos.h>

extern int devcode;             /* error code returned from crit err hndlr */
union REGS inregs, outregs;

main(argc,argv)
int argc;
char *argv[];
{
        int handle;
        int cnt,i;
        char buf[2];
        char cname[8];

        if (argc < 2)           /* if comm port not enter in command line */
        {
                fprintf(stderr," Port name: ");
                gets(cname);                            /* get comm port */
        }
        else    sprintf(cname,"%s",argv[1]);
        system("cls");                                  /* clear screen */

        if ((handle = open(cname,O_RDWR|O_BINARY)) == -1)  /* open port */
        {
                fprintf(stderr,"Can not open %s\n",argv[1]);
                exit(0);
        }

        /* get current ioctl settings */
        inregs.h.ah = 0x44;
        inregs.h.al = 0;
        inregs.x.bx = handle;
        inregs.x.dx = 0;
        intdos(&inregs,&outregs);

        /* set ioctl setting for binary mode */
        inregs.h.ah = 0x44;
        inregs.h.al = 1;
        inregs.x.bx = handle;
        inregs.h.dh = 0;
        inregs.h.dl = outregs.h.dl | 0x20;   /* set for binary mode */
        intdos(&inregs,&outregs);            /* set new ioctl settings */

        /* flush the transmit buffer */
        inregs.h.ah = 0x44;
        inregs.h.al = 3;
        inregs.x.bx = handle;
        inregs.x.cx = 1;
        i = 1;
        inregs.x.dx = (int) &i;
        intdos(&inregs,&outregs);           /* set new ioctl settings */

        /* flush receive buffer */
        inregs.h.ah = 0x44;
        inregs.h.al = 3;
        inregs.x.bx = handle;
        inregs.x.cx = 1;
        i = 2;
        inregs.x.dx = (int) &i;
        intdos(&inregs,&outregs);           /* set new ioctl settings */

        getsetcrit();                       /* set own critical error hndlr */
        errorcheck(handle);                 /* initial error check */
        while (0 == 0)                      /* infinite loop */
        {
                /* attempt to read char from device and write to screen */
                if ((cnt = read(handle,buf,1)) == 1)    /* got a char */
                        putc(buf[0],stderr);
                errorcheck(handle);

                /* attempt to read char from keyboard and write to device */
                if ((bdos(11,0,0)&0377) == 0xFF)        /* if char waiting */
                {
                        buf[0] = bdos(8,0,0) & 0377;    /* read keybd char */
                        if ((buf[0] == NULL) &&         /* 2 char key */
                        ((bdos(11,0,0)&0377) == 0xff))
                        {
                                buf[1] = bdos(8,0,0) & 0377;  /* next */
                                if (buf[1] == 0x44)           /* F10 = quit */
                                        break;
                                else    write(handle,buf,2);  /* write */
                        }
                        else    write(handle,buf,1);          /* write */
                        errorcheck(handle);
                }
        }
        close(handle);                                  /* close port */
        restorecrit();                  /* back to DOS critical error hndlr */
}

/*
        errorcheck() makes an IOCTL read call to see if there are any
        errors.  The read will automatically clear the error flags.  If
        there was an error, report what type and continue.
*/

errorcheck(handle)
int handle;
{
        char buf[7];

        if (devcode == 0x0a)                    /* write fault */
                fprintf(stderr,"[TBOE]");       /* transmit buf overrun */
        else
        {
                inregs.h.ah = 0x44;
                inregs.h.al = 0x02;                 /* read ioctl string */
                inregs.x.bx = handle;               /* file handle */
                inregs.x.cx = 0x07;                 /* number of bytes to read*/
                inregs.x.dx = buf;                  /* where to put string */
                if (intdos(&inregs,&outregs) != 7)  /* unsuccessful read */
                {
                        fprintf(stderr,"IOCTL read error\n");
                        devcode = 0xff;
                        return;
                }

                if ((buf[0] != 0) || (buf[1] != 0))     /* ERROR!!! */
                {
                        if (buf[0] & 0x02)      /* overrun error */
                                fprintf(stderr," Overrun error\n ");
                        if (buf[0] & 0x04)      /* parity error */
                                fprintf(stderr," Parity error\n ");
                        if (buf[0] & 0x08)      /* framing error */
                                fprintf(stderr," Framing error\n ");
                        if (buf[0] & 0x10)      /* break interrupt */
                                fprintf(stderr," Break interrupt\n ");
                        fprintf(stderr," Modem status = %x hex\n",buf[6]);
                }
        }
        devcode = 0xff;                 /* 'clear' out error number */
}
