;   File: TERMERR.ASM
;   Project: Terminal program in "C" for Comtrol Controllers
;   Date: 18 October 1990
;   Comments: this file provides the code to install our own critical error
;             handler during TERM1, to restore the original critical error
;             handler when exiting TERM1, and the critical error routine
;             itself.  The routine save the error number in DEVCODE and
;             returns a 3 for an automatic fail.

_text   segment byte public 'code'
_text   ends
_data   segment word public 'data'
_data   ends
const   segment word public 'const'
const   ends
_bss    segment word public 'bss'
_bss    ends

dgroup  group   const, _bss, _data
        assume  cs: _text, ds: dgroup, ss: dgroup, es: dgroup

_data   segment

int24h_offset   dw      ?       ; critical error handler interrupt vector
int24h_segment  dw      ?

        public  _devcode
_devcode        dw      ?       ; device driver error code

_data   ends

;----------------------------------------------------------------------------

_text   segment

        public  _restorecrit
        public  _getsetcrit
        public  _criterr
;----------------------------------------------------------------------------
;   name: _getsetcrit
;   function: gets the current address of the critical error handler
;             (int 24h) and saves it to be restored later on exiting TERM1.
;             Replaces the address with the address of the routine _criterr
;             which is our own routine to handle critical errors so that the
;             DOS question 'Abort, Retry, Fail' does not come up on an error.
;
;   enter: no parameters on entrance
;   exit:  no parameters on exit

_getsetcrit     proc
        push    bp                      ; prolog
        mov     bp,sp
        push    ds

        mov     ax,_data                ; establish data segment
        mov     ds,ax

        mov     al,24h                  ; AL = interrupt #
        mov     ah,35h                  ; AH = function = get vector
        int     21h
        mov     int24h_offset,bx        ; save current critical error
        mov     int24h_segment,es       ; interrupt vector

        mov     dx,offset _criterr      ; DS:DX=> new critical error
        mov     ax,cs                   ; interrupt vector
        mov     ds,ax
        mov     al,24h                  ; AL = interrupt #
        mov     ah,25h                  ; AH = function = set interrupt vector
        int     21h

        pop     ds                      ; epilog
        mov     sp,bp
        pop     bp
        ret
_getsetcrit     endp

;----------------------------------------------------------------------------
;   name: _restorecrit
;   function: to restore the original critical error routine address
;             when exiting TERM.
;   enter: no parameters on entrance
;   exit:  no parameters on exit

_restorecrit    proc
        push    bp
        mov     bp,sp
        push    ds

        mov     dx,int24h_offset
        mov     ax,int24h_segment
        mov     ds,ax
        mov     al,24h
        mov     ah,25h
        int     21h

        pop     ds
        mov     sp,bp
        pop     bp
        ret
_restorecrit    endp

;----------------------------------------------------------------------------
;   name: _criterr
;   function: interrupt routine to handle a critical error without asking
;             for an operator response (Abort, Fail, Retry).
;             The error number is stored in _devcode and 3 is returned
;             to automatically fail the process.
;   enter: no parameters on entrance
;   exit:  no parameters on exit

_criterr        proc
        push    bp                      ; prolog
        mov     bp,sp
        push    ds

        mov     ax,_data                ; and setup local data segment
        mov     ds,ax
        assume  ds:_data

        mov     ax,di                   ; get error code
        and     ax,0ffh                 ; mask off top byte
        mov     _devcode,ax             ; and save it locally
        mov     al,3                    ; fail the system call in progress

        pop     ds                      ; epilog
        mov     sp,bp
        pop     bp
        iret
_criterr        endp

;----------------------------------------------------------------------------

_text   ends
end
